@@ -6,21 +6,15 @@
 
 use crate::polynomial::Polynomial;
 use nalgebra::{ComplexField, RealField};
-use num_traits::FromPrimitive;
 
 #[derive(Debug, Clone)]
-pub struct CubicSpline<N: ComplexField + FromPrimitive + Copy>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+pub struct CubicSpline<N: ComplexField> {
     cubics: Vec<Polynomial<N>>,
     ranges: Vec<(N::RealField, N::RealField)>,
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> CubicSpline<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> CubicSpline<N> {
     pub fn evaluate(&self, x: N::RealField) -> Result<N, String> {
         if self.cubics.is_empty() {
             return Err("CubicSpline evaluate: Empty spline".to_owned());
@@ -79,14 +73,11 @@ where
 ///     }
 /// }
 /// ```
-pub fn spline_free<N: ComplexField + FromPrimitive + Copy>(
+pub fn spline_free<N: ComplexField>(
     xs: &[N::RealField],
     ys: &[N],
     tol: N::RealField,
-) -> Result<CubicSpline<N>, String>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+) -> Result<CubicSpline<N>, String> {
     if xs.len() != ys.len() {
         return Err("spline_free: xs and ys must be same length".to_owned());
     }
@@ -198,15 +189,12 @@ where
 ///     }
 /// }
 /// ```
-pub fn spline_clamped<N: ComplexField + FromPrimitive + Copy>(
+pub fn spline_clamped<N: ComplexField>(
     xs: &[N::RealField],
     ys: &[N],
     (f_0, f_n): (N, N),
     tol: N::RealField,
-) -> Result<CubicSpline<N>, String>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+) -> Result<CubicSpline<N>, String> {
     if xs.len() != ys.len() {
         return Err("spline_clamped: xs and ys must be same length".to_owned());
     }
