@@ -5,7 +5,9 @@
  */
 
 use super::{IVPSolver, IVPStatus};
-use nalgebra::{ComplexField, RealField, SVector};
+use nalgebra::{
+    allocator::Allocator, ComplexField, DefaultAllocator, DimName, RealField, VectorN, U3, U5,
+};
 use num_traits::{FromPrimitive, Zero};
 use std::collections::VecDeque;
 
@@ -16,25 +18,27 @@ use std::collections::VecDeque;
 ///
 /// # Examples
 /// See `struct Adams` for an example of implementing this trait
-pub trait AdamsSolver<N, const S: usize, const O: usize>: Sized
+pub trait AdamsSolver<N: ComplexField, S: DimName, O: DimName>: Sized
 where
-    N: ComplexField,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N::RealField, O>,
 {
     /// The polynomial interpolation coefficients for the predictor. Should start
     /// with the coefficient for n - 1
-    fn predictor_coefficients() -> SVector<N::RealField, O>;
+    fn predictor_coefficients() -> VectorN<N::RealField, O>;
     /// The polynomial interpolation coefficients for the corrector. Must be
     /// the same length as predictor_coefficients. Should start with the
     /// implicit coefficient.
-    fn corrector_coefficients() -> SVector<N::RealField, O>;
+    fn corrector_coefficients() -> VectorN<N::RealField, O>;
     /// Coefficient for multiplying error by.
     fn error_coefficient() -> N::RealField;
 
     /// Use AdamsInfo to solve an initial value problem
-    fn solve_ivp<T, F>(self, f: F, params: &mut T) -> super::Path<N, N::RealField, S>
-    where
-        T: Clone,
-        F: FnMut(N::RealField, &[N], &mut T) -> Result<SVector<N, S>, String>;
+    fn solve_ivp<T: Clone, F: FnMut(N::RealField, &[N], &mut T) -> Result<VectorN<N, S>, String>>(
+        self,
+        f: F,
+        params: &mut T,
+    ) -> super::Path<N, N::RealField, S>;
 
     /// Set the error tolerance for this solver.
     fn with_tolerance(self, tol: N::RealField) -> Result<Self, String>;
@@ -56,31 +60,32 @@ where
 /// the predictor and correctorr coefficients. It is up to the AdamsSolver
 /// to set up AdamsInfo with the correct coefficients.
 #[derive(Debug, Clone)]
-pub struct AdamsInfo<N, const S: usize, const O: usize>
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+pub struct AdamsInfo<N: ComplexField, S: DimName, O: DimName>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, O>,
 {
     dt: Option<N::RealField>,
     time: Option<N::RealField>,
     end: Option<N::RealField>,
-    state: Option<SVector<N, S>>,
+    state: Option<VectorN<N, S>>,
     dt_max: Option<N::RealField>,
     dt_min: Option<N::RealField>,
     tolerance: Option<N::RealField>,
-    predictor_coefficients: SVector<N, O>,
-    corrector_coefficients: SVector<N, O>,
+    predictor_coefficients: VectorN<N, O>,
+    corrector_coefficients: VectorN<N, O>,
     error_coefficient: N::RealField,
-    memory: VecDeque<SVector<N, S>>,
-    states: VecDeque<(N::RealField, SVector<N, S>)>,
+    memory: VecDeque<VectorN<N, S>>,
+    states: VecDeque<(N::RealField, VectorN<N, S>)>,
     nflag: bool,
     last: bool,
 }
 
-impl<N, const S: usize, const O: usize> AdamsInfo<N, S, O>
+impl<N: ComplexField, S: DimName, O: DimName> AdamsInfo<N, S, O>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, O>,
 {
     pub fn new() -> Self {
         AdamsInfo {
@@ -91,8 +96,8 @@ where
             dt_max: None,
             dt_min: None,
             tolerance: None,
-            predictor_coefficients: SVector::<N, O>::zero(),
-            corrector_coefficients: SVector::<N, O>::zero(),
+            predictor_coefficients: VectorN::<N, O>::zero(),
+            corrector_coefficients: VectorN::<N, O>::zero(),
             error_coefficient: N::RealField::zero(),
             memory: VecDeque::new(),
             states: VecDeque::new(),
@@ -103,23 +108,25 @@ where
 }
 
 #[allow(clippy::too_many_arguments)]
-fn rk4<N, T, F, const S: usize>(
+fn rk4<
+    N: ComplexField,
+    S: DimName,
+    T: Clone,
+    F: FnMut(N::RealField, &[N], &mut T) -> Result<VectorN<N, S>, String>,
+>(
     time: N::RealField,
     dt: N::RealField,
     initial: &[N],
-    states: &mut VecDeque<(N::RealField, SVector<N, S>)>,
-    derivs: &mut VecDeque<SVector<N, S>>,
+    states: &mut VecDeque<(N::RealField, VectorN<N, S>)>,
+    derivs: &mut VecDeque<VectorN<N, S>>,
     mut f: F,
     params: &mut T,
     num: usize,
 ) -> Result<(), String>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-    T: Clone,
-    F: FnMut(N::RealField, &[N], &mut T) -> Result<SVector<N, S>, String>,
+    DefaultAllocator: Allocator<N, S>,
 {
-    let mut state: SVector<N, S> = SVector::from_column_slice(initial);
+    let mut state = VectorN::from_column_slice(initial);
     let mut time = time;
     for i in 0..num {
         let k1 = f(time, state.as_slice(), &mut params.clone())? * N::from_real(dt);
@@ -151,26 +158,26 @@ where
     Ok(())
 }
 
-impl<N, const S: usize, const O: usize> Default for AdamsInfo<N, S, O>
+impl<N: ComplexField, S: DimName, O: DimName> Default for AdamsInfo<N, S, O>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, O>,
 {
     fn default() -> Self {
         Self::new()
     }
 }
 
-impl<N, const S: usize, const O: usize> IVPSolver<N, S> for AdamsInfo<N, S, O>
+impl<N: ComplexField, S: DimName, O: DimName> IVPSolver<N, S> for AdamsInfo<N, S, O>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, O>,
 {
-    fn step<T, F>(&mut self, mut f: F, params: &mut T) -> Result<IVPStatus<N, S>, String>
-    where
-        T: Clone,
-        F: FnMut(N::RealField, &[N], &mut T) -> Result<SVector<N, S>, String>,
-    {
+    fn step<T: Clone, F: FnMut(N::RealField, &[N], &mut T) -> Result<VectorN<N, S>, String>>(
+        &mut self,
+        mut f: F,
+        params: &mut T,
+    ) -> Result<IVPStatus<N, S>, String> {
         if self.time.unwrap() >= self.end.unwrap() {
             return Ok(IVPStatus::Done);
         }
@@ -221,7 +228,7 @@ where
 
         let wp = &self.state.as_ref().unwrap();
         let wp =
-            SVector::<N, S>::from_iterator(wp.as_slice().iter().enumerate().map(|(ind, y)| {
+            VectorN::<N, S>::from_iterator(wp.as_slice().iter().enumerate().map(|(ind, y)| {
                 let mut acc = *y;
                 let dt = N::from_real(self.dt.unwrap());
                 for (j, mem) in self.memory.iter().enumerate() {
@@ -239,7 +246,7 @@ where
         )?;
         let wc = &self.state.as_ref().unwrap();
         let wc =
-            SVector::<N, S>::from_iterator(wc.as_slice().iter().enumerate().map(|(ind, y)| {
+            VectorN::<N, S>::from_iterator(wc.as_slice().iter().enumerate().map(|(ind, y)| {
                 let dt = N::from_real(self.dt.unwrap());
                 let mut acc = implicit[ind] * self.corrector_coefficients[0] * dt;
                 for (j, mem) in self.memory.iter().enumerate() {
@@ -384,7 +391,7 @@ where
     }
 
     fn with_initial_conditions(mut self, start: &[N]) -> Result<Self, String> {
-        self.state = Some(SVector::<N, S>::from_column_slice(start));
+        self.state = Some(VectorN::<N, S>::from_column_slice(start));
         Ok(self)
     }
 
@@ -392,7 +399,7 @@ where
         self
     }
 
-    fn get_initial_conditions(&self) -> Option<SVector<N, S>> {
+    fn get_initial_conditions(&self) -> Option<VectorN<N, S>> {
         if let Some(state) = &self.state {
             Some(state.clone())
         } else {
@@ -434,10 +441,10 @@ where
 ///
 /// # Examples
 /// ```
-/// use nalgebra::SVector;
+/// use nalgebra::{VectorN, U1};
 /// use bacon_sci::ivp::{Adams, AdamsSolver};
-/// fn derivatives(_t: f64, state: &[f64], _p: &mut ()) -> Result<SVector<f64, 1>, String> {
-///     Ok(SVector::<f64, 1>::from_column_slice(state))
+/// fn derivatives(_t: f64, state: &[f64], _p: &mut ()) -> Result<VectorN<f64, U1>, String> {
+///     Ok(VectorN::<f64, U1>::from_column_slice(state))
 /// }
 ///
 ///
@@ -458,27 +465,30 @@ where
 /// }
 /// ```
 #[derive(Debug, Clone)]
-pub struct Adams<N, const S: usize>
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+pub struct Adams<N: ComplexField, S: DimName>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N::RealField, U5>,
+    DefaultAllocator: Allocator<N, U5>,
 {
-    info: AdamsInfo<N, S, 5>,
+    info: AdamsInfo<N, S, U5>,
 }
 
-impl<N, const S: usize> Adams<N, S>
+impl<N: ComplexField, S: DimName> Adams<N, S>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U5>,
+    DefaultAllocator: Allocator<N::RealField, U5>,
 {
     pub fn new() -> Self {
         let mut info = AdamsInfo::new();
-        info.corrector_coefficients = SVector::<N, 5>::from_iterator(
+        info.corrector_coefficients = VectorN::<N, U5>::from_iterator(
             Self::corrector_coefficients()
                 .iter()
                 .map(|&x| N::from_real(x)),
         );
-        info.predictor_coefficients = SVector::<N, 5>::from_iterator(
+        info.predictor_coefficients = VectorN::<N, U5>::from_iterator(
             Self::predictor_coefficients()
                 .iter()
                 .map(|&x| N::from_real(x)),
@@ -489,23 +499,25 @@ where
     }
 }
 
-impl<N, const S: usize> Default for Adams<N, S>
+impl<N: ComplexField, S: DimName> Default for Adams<N, S>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N::RealField, U5>,
+    DefaultAllocator: Allocator<N, U5>,
 {
     fn default() -> Self {
         Self::new()
     }
 }
 
-impl<N, const S: usize> AdamsSolver<N, S, 5> for Adams<N, S>
+impl<N: ComplexField, S: DimName> AdamsSolver<N, S, U5> for Adams<N, S>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N::RealField, U5>,
+    DefaultAllocator: Allocator<N, U5>,
 {
-    fn predictor_coefficients() -> SVector<N::RealField, 5> {
-        SVector::<N::RealField, 5>::from_column_slice(&[
+    fn predictor_coefficients() -> VectorN<N::RealField, U5> {
+        VectorN::<N::RealField, U5>::from_column_slice(&[
             N::RealField::from_f64(55.0 / 24.0).unwrap(),
             N::RealField::from_f64(-59.0 / 24.0).unwrap(),
             N::RealField::from_f64(37.0 / 24.0).unwrap(),
@@ -514,8 +526,8 @@ where
         ])
     }
 
-    fn corrector_coefficients() -> SVector<N::RealField, 5> {
-        SVector::<N::RealField, 5>::from_column_slice(&[
+    fn corrector_coefficients() -> VectorN<N::RealField, U5> {
+        VectorN::<N::RealField, U5>::from_column_slice(&[
             N::RealField::from_f64(251.0 / 720.0).unwrap(),
             N::RealField::from_f64(646.0 / 720.0).unwrap(),
             N::RealField::from_f64(-264.0 / 720.0).unwrap(),
@@ -530,7 +542,7 @@ where
 
     fn solve_ivp<
         T: Clone,
-        F: FnMut(N::RealField, &[N], &mut T) -> Result<SVector<N, S>, String>,
+        F: FnMut(N::RealField, &[N], &mut T) -> Result<VectorN<N, S>, String>,
     >(
         self,
         f: F,
@@ -575,12 +587,13 @@ where
     }
 }
 
-impl<N, const S: usize> From<Adams<N, S>> for AdamsInfo<N, S, 5>
+impl<N: ComplexField, S: DimName> From<Adams<N, S>> for AdamsInfo<N, S, U5>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U5>,
+    DefaultAllocator: Allocator<N::RealField, U5>,
 {
-    fn from(adams: Adams<N, S>) -> AdamsInfo<N, S, 5> {
+    fn from(adams: Adams<N, S>) -> AdamsInfo<N, S, U5> {
         adams.info
     }
 }
@@ -592,10 +605,10 @@ where
 ///
 /// # Examples
 /// ```
-/// use nalgebra::SVector;
+/// use nalgebra::{VectorN, U1};
 /// use bacon_sci::ivp::{Adams2, AdamsSolver};
-/// fn derivatives(_t: f64, state: &[f64], _p: &mut ()) -> Result<SVector<f64, 1>, String> {
-///     Ok(SVector::<f64, 1>::from_column_slice(state))
+/// fn derivatives(_t: f64, state: &[f64], _p: &mut ()) -> Result<VectorN<f64, U1>, String> {
+///     Ok(VectorN::<f64, U1>::from_column_slice(state))
 /// }
 ///
 ///
@@ -616,27 +629,30 @@ where
 /// }
 /// ```
 #[derive(Debug, Clone)]
-pub struct Adams2<N, const S: usize>
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+pub struct Adams2<N: ComplexField, S: DimName>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N::RealField, U3>,
+    DefaultAllocator: Allocator<N, U3>,
 {
-    info: AdamsInfo<N, S, 3>,
+    info: AdamsInfo<N, S, U3>,
 }
 
-impl<N, const S: usize> Adams2<N, S>
+impl<N: ComplexField, S: DimName> Adams2<N, S>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U3>,
+    DefaultAllocator: Allocator<N::RealField, U3>,
 {
     pub fn new() -> Self {
         let mut info = AdamsInfo::new();
-        info.corrector_coefficients = SVector::<N, 3>::from_iterator(
+        info.corrector_coefficients = VectorN::<N, U3>::from_iterator(
             Self::corrector_coefficients()
                 .iter()
                 .map(|&x| N::from_real(x)),
         );
-        info.predictor_coefficients = SVector::<N, 3>::from_iterator(
+        info.predictor_coefficients = VectorN::<N, U3>::from_iterator(
             Self::predictor_coefficients()
                 .iter()
                 .map(|&x| N::from_real(x)),
@@ -647,31 +663,33 @@ where
     }
 }
 
-impl<N, const S: usize> Default for Adams2<N, S>
+impl<N: ComplexField, S: DimName> Default for Adams2<N, S>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N::RealField, U3>,
+    DefaultAllocator: Allocator<N, U3>,
 {
     fn default() -> Self {
         Self::new()
     }
 }
 
-impl<N, const S: usize> AdamsSolver<N, S, 3> for Adams2<N, S>
+impl<N: ComplexField, S: DimName> AdamsSolver<N, S, U3> for Adams2<N, S>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N::RealField, U3>,
+    DefaultAllocator: Allocator<N, U3>,
 {
-    fn predictor_coefficients() -> SVector<N::RealField, 3> {
-        SVector::<N::RealField, 3>::from_column_slice(&[
+    fn predictor_coefficients() -> VectorN<N::RealField, U3> {
+        VectorN::<N::RealField, U3>::from_column_slice(&[
             N::RealField::from_f64(1.5).unwrap(),
             N::RealField::from_f64(-0.5).unwrap(),
             N::RealField::zero(),
         ])
     }
 
-    fn corrector_coefficients() -> SVector<N::RealField, 3> {
-        SVector::<N::RealField, 3>::from_column_slice(&[
+    fn corrector_coefficients() -> VectorN<N::RealField, U3> {
+        VectorN::<N::RealField, U3>::from_column_slice(&[
             N::RealField::from_f64(5.0 / 12.0).unwrap(),
             N::RealField::from_f64(2.0 / 3.0).unwrap(),
             N::RealField::from_f64(-1.0 / 12.0).unwrap(),
@@ -684,7 +702,7 @@ where
 
     fn solve_ivp<
         T: Clone,
-        F: FnMut(N::RealField, &[N], &mut T) -> Result<SVector<N, S>, String>,
+        F: FnMut(N::RealField, &[N], &mut T) -> Result<VectorN<N, S>, String>,
     >(
         self,
         f: F,
@@ -729,12 +747,13 @@ where
     }
 }
 
-impl<N, const S: usize> From<Adams2<N, S>> for AdamsInfo<N, S, 3>
+impl<N: ComplexField, S: DimName> From<Adams2<N, S>> for AdamsInfo<N, S, U3>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U3>,
+    DefaultAllocator: Allocator<N::RealField, U3>,
 {
-    fn from(adams: Adams2<N, S>) -> AdamsInfo<N, S, 3> {
+    fn from(adams: Adams2<N, S>) -> AdamsInfo<N, S, U3> {
         adams.info
     }
 }
