@@ -5,7 +5,10 @@
  */
 
 use super::{IVPSolver, IVPStatus};
-use nalgebra::{ComplexField, RealField, SMatrix, SVector};
+use nalgebra::{
+    allocator::Allocator, ComplexField, DefaultAllocator, DimName, MatrixMN, RealField, VectorN,
+    U4, U6,
+};
 use num_traits::{FromPrimitive, Zero};
 
 /// This trait allows a struct to be used in the Runge-Kutta solver.
@@ -16,34 +19,36 @@ use num_traits::{FromPrimitive, Zero};
 ///
 /// # Examples
 /// See `struct RK45` for an example of implementing this trait
-pub trait RungeKuttaSolver<N, const S: usize, const O: usize>: Sized
+pub trait RungeKuttaSolver<N: ComplexField, S: DimName, O: DimName>: Sized
 where
-    N: ComplexField,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N::RealField, O>,
+    DefaultAllocator: Allocator<N::RealField, O, O>,
 {
     /// Returns a vec of coeffecients to multiply the time step by when getting
     /// intermediate results. Upper-left portion of Butch Tableaux
-    fn t_coefficients() -> SVector<N::RealField, O>;
+    fn t_coefficients() -> VectorN<N::RealField, O>;
 
     /// Returns the coefficients to use on the k_i's when finding another
     /// k_i. Upper-right portion of the Butch Tableax. Should be
     /// an NxN-1 matrix, where N is the order of the Runge-Kutta Method (Or order+1 for
     /// adaptive methods)
-    fn k_coefficients() -> SMatrix<N::RealField, O, O>;
+    fn k_coefficients() -> MatrixMN<N::RealField, O, O>;
 
     /// Coefficients to use when calculating the final step to take.
     /// These are the weights of the weighted average of k_i's. Bottom
     /// portion of the Butch Tableaux. For adaptive methods, this is the first
     /// row of the bottom portion.
-    fn avg_coefficients() -> SVector<N::RealField, O>;
+    fn avg_coefficients() -> VectorN<N::RealField, O>;
 
     /// Coefficients to use on
     /// the k_i's to find the error between the two orders
     /// of Runge-Kutta methods. In the Butch Tableaux, this is
     /// the first row of the bottom portion minus the second row.
-    fn error_coefficients() -> SVector<N::RealField, O>;
+    fn error_coefficients() -> VectorN<N::RealField, O>;
 
     /// Ideally, call RKInfo.solve_ivp
-    fn solve_ivp<T: Clone, F: FnMut(N::RealField, &[N], &mut T) -> Result<SVector<N, S>, String>>(
+    fn solve_ivp<T: Clone, F: FnMut(N::RealField, &[N], &mut T) -> Result<VectorN<N, S>, String>>(
         self,
         f: F,
         params: &mut T,
@@ -69,28 +74,31 @@ where
 /// based entirely on the Butch Tableaux coefficients. It is up
 /// to the RungeKuttaSolver to set up RKInfo. See RK45 for an example.
 #[derive(Debug, Clone)]
-pub struct RKInfo<N: ComplexField + FromPrimitive, const S: usize, const O: usize>
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+pub struct RKInfo<N: ComplexField, S: DimName, O: DimName>
 where
-    N: ComplexField + FromPrimitive,
-    <N as ComplexField>::RealField: FromPrimitive,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, O>,
+    DefaultAllocator: Allocator<N, O, O>,
 {
     dt: Option<N::RealField>,
     time: Option<N::RealField>,
     end: Option<N::RealField>,
-    state: Option<SVector<N, S>>,
+    state: Option<VectorN<N, S>>,
     dt_max: Option<N::RealField>,
     dt_min: Option<N::RealField>,
     tolerance: Option<N::RealField>,
-    a_coefficients: SVector<N, O>,
-    k_coefficients: SMatrix<N, O, O>,
-    avg_coefficients: SVector<N, O>,
-    error_coefficients: SVector<N, O>,
+    a_coefficients: VectorN<N, O>,
+    k_coefficients: MatrixMN<N, O, O>,
+    avg_coefficients: VectorN<N, O>,
+    error_coefficients: VectorN<N, O>,
 }
 
-impl<N: ComplexField + FromPrimitive, const S: usize, const O: usize> RKInfo<N, S, O>
+impl<N: ComplexField, S: DimName, O: DimName> RKInfo<N, S, O>
 where
-    N: ComplexField + FromPrimitive,
-    <N as ComplexField>::RealField: FromPrimitive,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, O>,
+    DefaultAllocator: Allocator<N, O, O>,
 {
     fn new() -> Self {
         RKInfo {
@@ -101,24 +109,26 @@ where
             dt_max: None,
             dt_min: None,
             tolerance: None,
-            a_coefficients: SVector::zero(),
-            k_coefficients: SMatrix::zero(),
-            avg_coefficients: SVector::zero(),
-            error_coefficients: SVector::zero(),
+            a_coefficients: VectorN::zero(),
+            k_coefficients: MatrixMN::zero(),
+            avg_coefficients: VectorN::zero(),
+            error_coefficients: VectorN::zero(),
         }
     }
 }
 
-impl<N, const S: usize, const O: usize> IVPSolver<N, S> for RKInfo<N, S, O>
+impl<N: ComplexField, S: DimName, O: DimName> IVPSolver<N, S> for RKInfo<N, S, O>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, O>,
+    DefaultAllocator: Allocator<N, O, O>,
+    DefaultAllocator: Allocator<N, S, O>,
 {
-    fn step<T, F>(&mut self, mut f: F, params: &mut T) -> Result<IVPStatus<N, S>, String>
-    where
-        T: Clone,
-        F: FnMut(N::RealField, &[N], &mut T) -> Result<SVector<N, S>, String>,
-    {
+    fn step<T: Clone, F: FnMut(N::RealField, &[N], &mut T) -> Result<VectorN<N, S>, String>>(
+        &mut self,
+        mut f: F,
+        params: &mut T,
+    ) -> Result<IVPStatus<N, S>, String> {
         if self.time.unwrap() >= self.end.unwrap() {
             return Ok(IVPStatus::Done);
         }
@@ -136,10 +146,10 @@ where
 
         let num_k = self.k_coefficients.row(0).len();
 
-        let mut half_steps: SMatrix<N, S, O> = SMatrix::zero();
+        let mut half_steps: MatrixMN<N, S, O> = MatrixMN::zero();
         let num_s = half_steps.column(0).len();
         for i in 0..num_k {
-            let state = SVector::<N, S>::from_iterator(
+            let state = VectorN::<N, S>::from_iterator(
                 self.state
                     .as_ref()
                     .unwrap()
@@ -164,7 +174,7 @@ where
             }
         }
 
-        let error_vec = SVector::<N, S>::from_iterator(
+        let error_vec = VectorN::<N, S>::from_iterator(
             half_steps.column(0).iter().enumerate().map(|(ind, y)| {
                 let mut acc = *y * self.error_coefficients[0];
                 for j in 1..num_k {
@@ -183,7 +193,7 @@ where
             for ind in 1..half_steps.row(0).len() {
                 state += &half_steps.column(ind) * self.avg_coefficients[ind];
             }*/
-            let state = SVector::<N, S>::from_iterator(
+            let state = VectorN::<N, S>::from_iterator(
                 self.state
                     .as_ref()
                     .unwrap()
@@ -284,7 +294,7 @@ where
     }
 
     fn with_initial_conditions(mut self, start: &[N]) -> Result<Self, String> {
-        self.state = Some(SVector::<N, S>::from_column_slice(start));
+        self.state = Some(VectorN::<N, S>::from_column_slice(start));
         Ok(self)
     }
 
@@ -292,7 +302,7 @@ where
         self
     }
 
-    fn get_initial_conditions(&self) -> Option<SVector<N, S>> {
+    fn get_initial_conditions(&self) -> Option<VectorN<N, S>> {
         if let Some(state) = &self.state {
             Some(state.clone())
         } else {
@@ -335,10 +345,10 @@ where
 ///
 /// # Examples
 /// ```
-/// use nalgebra::SVector;
+/// use nalgebra::{VectorN, U1};
 /// use bacon_sci::ivp::{RK45, RungeKuttaSolver};
-/// fn derivatives(_t: f64, state: &[f64], _p: &mut ()) -> Result<SVector<f64, 1>, String> {
-///     Ok(SVector::<f64, 1>::from_column_slice(state))
+/// fn derivatives(_t: f64, state: &[f64], _p: &mut ()) -> Result<VectorN<f64, U1>, String> {
+///     Ok(VectorN::<f64, U1>::from_column_slice(state))
 /// }
 ///
 /// fn example() -> Result<(), String> {
@@ -358,41 +368,50 @@ where
 /// }
 /// ```
 #[derive(Debug, Clone)]
-pub struct RK45<N, const S: usize>
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+pub struct RK45<N: ComplexField, S: DimName>
 where
-    N: ComplexField + FromPrimitive,
-    <N as ComplexField>::RealField: FromPrimitive,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U6>,
+    DefaultAllocator: Allocator<N, U6, U6>,
+    DefaultAllocator: Allocator<N, S, U6>,
+    DefaultAllocator: Allocator<N::RealField, U6>,
+    DefaultAllocator: Allocator<N::RealField, U6, U6>,
 {
-    info: RKInfo<N, S, 6>,
+    info: RKInfo<N, S, U6>,
 }
 
-impl<N, const S: usize> RK45<N, S>
+impl<N: ComplexField, S: DimName> RK45<N, S>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U6>,
+    DefaultAllocator: Allocator<N, U6, U6>,
+    DefaultAllocator: Allocator<N, S, U6>,
+    DefaultAllocator: Allocator<N::RealField, U6>,
+    DefaultAllocator: Allocator<N::RealField, U6, U6>,
 {
     #[allow(clippy::new_without_default)]
     pub fn new() -> Self {
-        let mut info: RKInfo<N, S, 6> = RKInfo::new();
-        info.a_coefficients = SVector::<N, 6>::from_iterator(
+        let mut info: RKInfo<N, S, U6> = RKInfo::new();
+        info.a_coefficients = VectorN::<N, U6>::from_iterator(
             Self::t_coefficients()
                 .as_slice()
                 .iter()
                 .map(|x| N::from_real(*x)),
         );
-        info.k_coefficients = SMatrix::<N, 6, 6>::from_iterator(
+        info.k_coefficients = MatrixMN::<N, U6, U6>::from_iterator(
             Self::k_coefficients()
                 .as_slice()
                 .iter()
                 .map(|x| N::from_real(*x)),
         );
-        info.avg_coefficients = SVector::<N, 6>::from_iterator(
+        info.avg_coefficients = VectorN::<N, U6>::from_iterator(
             Self::avg_coefficients()
                 .as_slice()
                 .iter()
                 .map(|x| N::from_real(*x)),
         );
-        info.error_coefficients = SVector::<N, 6>::from_iterator(
+        info.error_coefficients = VectorN::<N, U6>::from_iterator(
             Self::error_coefficients()
                 .as_slice()
                 .iter()
@@ -402,13 +421,17 @@ where
     }
 }
 
-impl<N, const S: usize> RungeKuttaSolver<N, S, 6> for RK45<N, S>
+impl<N: ComplexField, S: DimName> RungeKuttaSolver<N, S, U6> for RK45<N, S>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U6>,
+    DefaultAllocator: Allocator<N, U6, U6>,
+    DefaultAllocator: Allocator<N::RealField, U6>,
+    DefaultAllocator: Allocator<N::RealField, U6, U6>,
+    DefaultAllocator: Allocator<N, S, U6>,
 {
-    fn t_coefficients() -> SVector<N::RealField, 6> {
-        SVector::<N::RealField, 6>::from_column_slice(&[
+    fn t_coefficients() -> VectorN<N::RealField, U6> {
+        VectorN::<N::RealField, U6>::from_column_slice(&[
             N::RealField::from_f64(0.0).unwrap(),
             N::RealField::from_f64(0.25).unwrap(),
             N::RealField::from_f64(3.0 / 8.0).unwrap(),
@@ -418,8 +441,8 @@ where
         ])
     }
 
-    fn k_coefficients() -> SMatrix<N::RealField, 6, 6> {
-        SMatrix::<N::RealField, 6, 6>::from_vec(vec![
+    fn k_coefficients() -> MatrixMN<N::RealField, U6, U6> {
+        MatrixMN::<N::RealField, U6, U6>::new(
             // Row 0
             N::RealField::zero(),
             N::RealField::zero(),
@@ -462,11 +485,11 @@ where
             N::RealField::from_f64(1859.0 / 4104.0).unwrap(),
             N::RealField::from_f64(-11.0 / 40.0).unwrap(),
             N::RealField::zero(),
-        ])
+        )
     }
 
-    fn avg_coefficients() -> SVector<N::RealField, 6> {
-        SVector::<N::RealField, 6>::from_column_slice(&[
+    fn avg_coefficients() -> VectorN<N::RealField, U6> {
+        VectorN::<N::RealField, U6>::from_column_slice(&[
             N::RealField::from_f64(25.0 / 216.0).unwrap(),
             N::RealField::from_f64(0.0).unwrap(),
             N::RealField::from_f64(1408.0 / 2565.0).unwrap(),
@@ -476,8 +499,8 @@ where
         ])
     }
 
-    fn error_coefficients() -> SVector<N::RealField, 6> {
-        SVector::<N::RealField, 6>::from_column_slice(&[
+    fn error_coefficients() -> VectorN<N::RealField, U6> {
+        VectorN::<N::RealField, U6>::from_column_slice(&[
             N::RealField::from_f64(1.0 / 360.0).unwrap(),
             N::RealField::from_f64(0.0).unwrap(),
             N::RealField::from_f64(-128.0 / 4275.0).unwrap(),
@@ -487,194 +510,9 @@ where
         ])
     }
 
-    fn solve_ivp<T, F>(self, f: F, params: &mut T) -> super::Path<N, N::RealField, S>
-    where
-        T: Clone,
-        F: FnMut(N::RealField, &[N], &mut T) -> Result<SVector<N, S>, String>,
-    {
-        self.info.solve_ivp(f, params)
-    }
-
-    fn with_tolerance(mut self, tol: N::RealField) -> Result<Self, String> {
-        self.info = self.info.with_tolerance(tol)?;
-        Ok(self)
-    }
-
-    fn with_dt_max(mut self, max: N::RealField) -> Result<Self, String> {
-        self.info = self.info.with_dt_max(max)?;
-        Ok(self)
-    }
-
-    fn with_dt_min(mut self, min: N::RealField) -> Result<Self, String> {
-        self.info = self.info.with_dt_min(min)?;
-        Ok(self)
-    }
-
-    fn with_start(mut self, t_initial: N::RealField) -> Result<Self, String> {
-        self.info = self.info.with_start(t_initial)?;
-        Ok(self)
-    }
-
-    fn with_end(mut self, t_final: N::RealField) -> Result<Self, String> {
-        self.info = self.info.with_end(t_final)?;
-        Ok(self)
-    }
-
-    fn with_initial_conditions(mut self, start: &[N]) -> Result<Self, String> {
-        self.info = self.info.with_initial_conditions(start)?;
-        Ok(self)
-    }
-
-    fn build(self) -> Self {
-        self
-    }
-}
-
-impl<N, const S: usize> From<RK45<N, S>> for RKInfo<N, S, 6>
-where
-    N: ComplexField + FromPrimitive,
-    <N as ComplexField>::RealField: FromPrimitive,
-{
-    fn from(rk: RK45<N, S>) -> RKInfo<N, S, 6> {
-        rk.info
-    }
-}
-
-/// Bogacki-Shampine method for solving an IVP.
-///
-/// Defines the Butch Tableaux for a 5(4) order adaptive
-/// runge-kutta method. Uses RKInfo to do the actual solving.
-/// Provides an interface for setting the conditions on RKInfo.
-///
-/// # Examples
-/// ```
-/// use nalgebra::SVector;
-/// use bacon_sci::ivp::{RK23, RungeKuttaSolver};
-/// fn derivatives(_t: f64, state: &[f64], _p: &mut ()) -> Result<SVector<f64, 1>, String> {
-///     Ok(SVector::<f64, 1>::from_column_slice(state))
-/// }
-///
-/// fn example() -> Result<(), String> {
-///     let rk45 = RK23::new()
-///         .with_dt_max(0.1)?
-///         .with_dt_min(0.001)?
-///         .with_start(0.0)?
-///         .with_end(10.0)?
-///         .with_tolerance(0.0001)?
-///         .with_initial_conditions(&[1.0])?
-///         .build();
-///     let path = rk45.solve_ivp(derivatives, &mut ())?;
-///     for (time, state) in &path {
-///         assert!((time.exp() - state.column(0)[0]).abs() < 0.001);
-///     }
-///     Ok(())
-/// }
-/// ```
-#[derive(Debug, Clone)]
-pub struct RK23<N, const S: usize>
-where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
-    info: RKInfo<N, S, 4>,
-}
-
-impl<N, const S: usize> RK23<N, S>
-where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
-    #[allow(clippy::new_without_default)]
-    pub fn new() -> Self {
-        let mut info: RKInfo<N, S, 4> = RKInfo::new();
-        info.a_coefficients = SVector::<N, 4>::from_iterator(
-            Self::t_coefficients()
-                .as_slice()
-                .iter()
-                .map(|x| N::from_real(*x)),
-        );
-        info.k_coefficients = SMatrix::<N, 4, 4>::from_iterator(
-            Self::k_coefficients()
-                .as_slice()
-                .iter()
-                .map(|x| N::from_real(*x)),
-        );
-        info.avg_coefficients = SVector::<N, 4>::from_iterator(
-            Self::avg_coefficients()
-                .as_slice()
-                .iter()
-                .map(|x| N::from_real(*x)),
-        );
-        info.error_coefficients = SVector::<N, 4>::from_iterator(
-            Self::error_coefficients()
-                .as_slice()
-                .iter()
-                .map(|x| N::from_real(*x)),
-        );
-        RK23 { info }
-    }
-}
-
-impl<N, const S: usize> RungeKuttaSolver<N, S, 4> for RK23<N, S>
-where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
-    fn t_coefficients() -> SVector<N::RealField, 4> {
-        SVector::<N::RealField, 4>::from_column_slice(&[
-            N::RealField::from_f64(0.0).unwrap(),
-            N::RealField::from_f64(0.5).unwrap(),
-            N::RealField::from_f64(0.75).unwrap(),
-            N::RealField::from_f64(1.0).unwrap(),
-        ])
-    }
-
-    fn k_coefficients() -> SMatrix<N::RealField, 4, 4> {
-        SMatrix::<N::RealField, 4, 4>::from_vec(vec![
-            // Row 0
-            N::RealField::zero(),
-            N::RealField::zero(),
-            N::RealField::zero(),
-            N::RealField::zero(),
-            // Row 1
-            N::RealField::from_f64(0.5).unwrap(),
-            N::RealField::zero(),
-            N::RealField::zero(),
-            N::RealField::zero(),
-            // Row 2
-            N::RealField::zero(),
-            N::RealField::from_f64(0.75).unwrap(),
-            N::RealField::zero(),
-            N::RealField::zero(),
-            // Row 3
-            N::RealField::from_f64(2.0 / 9.0).unwrap(),
-            N::RealField::from_f64(1.0 / 3.0).unwrap(),
-            N::RealField::from_f64(4.0 / 9.0).unwrap(),
-            N::RealField::zero(),
-        ])
-    }
-
-    fn avg_coefficients() -> SVector<N::RealField, 4> {
-        SVector::<N::RealField, 4>::from_column_slice(&[
-            N::RealField::from_f64(2.0 / 9.0).unwrap(),
-            N::RealField::from_f64(1.0 / 3.0).unwrap(),
-            N::RealField::from_f64(4.0 / 9.0).unwrap(),
-            N::RealField::zero(),
-        ])
-    }
-
-    fn error_coefficients() -> SVector<N::RealField, 4> {
-        SVector::<N::RealField, 4>::from_column_slice(&[
-            N::RealField::from_f64(2.0 / 9.0 - 7.0 / 24.0).unwrap(),
-            N::RealField::from_f64(1.0 / 3.0 - 0.25).unwrap(),
-            N::RealField::from_f64(4.0 / 9.0 - 1.0 / 3.0).unwrap(),
-            N::RealField::from_f64(-1.0 / 8.0).unwrap(),
-        ])
-    }
-
     fn solve_ivp<
         T: Clone,
-        F: FnMut(N::RealField, &[N], &mut T) -> Result<SVector<N, S>, String>,
+        F: FnMut(N::RealField, &[N], &mut T) -> Result<VectorN<N, S>, String>,
     >(
         self,
         f: F,
@@ -718,12 +556,221 @@ where
     }
 }
 
-impl<N, const S: usize> From<RK23<N, S>> for RKInfo<N, S, 4>
+impl<N: ComplexField, S: DimName> From<RK45<N, S>> for RKInfo<N, S, U6>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U6>,
+    DefaultAllocator: Allocator<N, S, U6>,
+    DefaultAllocator: Allocator<N, U6, U6>,
+    DefaultAllocator: Allocator<N::RealField, U6>,
+    DefaultAllocator: Allocator<N::RealField, U6>,
 {
-    fn from(rk: RK23<N, S>) -> RKInfo<N, S, 4> {
+    fn from(rk: RK45<N, S>) -> RKInfo<N, S, U6> {
+        rk.info
+    }
+}
+
+/// Bogacki-Shampine method for solving an IVP.
+///
+/// Defines the Butch Tableaux for a 5(4) order adaptive
+/// runge-kutta method. Uses RKInfo to do the actual solving.
+/// Provides an interface for setting the conditions on RKInfo.
+///
+/// # Examples
+/// ```
+/// use nalgebra::{VectorN, U1};
+/// use bacon_sci::ivp::{RK23, RungeKuttaSolver};
+/// fn derivatives(_t: f64, state: &[f64], _p: &mut ()) -> Result<VectorN<f64, U1>, String> {
+///     Ok(VectorN::<f64, U1>::from_column_slice(state))
+/// }
+///
+/// fn example() -> Result<(), String> {
+///     let rk45 = RK23::new()
+///         .with_dt_max(0.1)?
+///         .with_dt_min(0.001)?
+///         .with_start(0.0)?
+///         .with_end(10.0)?
+///         .with_tolerance(0.0001)?
+///         .with_initial_conditions(&[1.0])?
+///         .build();
+///     let path = rk45.solve_ivp(derivatives, &mut ())?;
+///     for (time, state) in &path {
+///         assert!((time.exp() - state.column(0)[0]).abs() < 0.001);
+///     }
+///     Ok(())
+/// }
+/// ```
+#[derive(Debug, Clone)]
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+pub struct RK23<N: ComplexField, S: DimName>
+where
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U4>,
+    DefaultAllocator: Allocator<N, U4, U4>,
+    DefaultAllocator: Allocator<N, S, U4>,
+    DefaultAllocator: Allocator<N::RealField, U4>,
+    DefaultAllocator: Allocator<N::RealField, U4, U4>,
+{
+    info: RKInfo<N, S, U4>,
+}
+
+impl<N: ComplexField, S: DimName> RK23<N, S>
+where
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U4>,
+    DefaultAllocator: Allocator<N, U4, U4>,
+    DefaultAllocator: Allocator<N, S, U4>,
+    DefaultAllocator: Allocator<N::RealField, U4>,
+    DefaultAllocator: Allocator<N::RealField, U4, U4>,
+{
+    #[allow(clippy::new_without_default)]
+    pub fn new() -> Self {
+        let mut info: RKInfo<N, S, U4> = RKInfo::new();
+        info.a_coefficients = VectorN::<N, U4>::from_iterator(
+            Self::t_coefficients()
+                .as_slice()
+                .iter()
+                .map(|x| N::from_real(*x)),
+        );
+        info.k_coefficients = MatrixMN::<N, U4, U4>::from_iterator(
+            Self::k_coefficients()
+                .as_slice()
+                .iter()
+                .map(|x| N::from_real(*x)),
+        );
+        info.avg_coefficients = VectorN::<N, U4>::from_iterator(
+            Self::avg_coefficients()
+                .as_slice()
+                .iter()
+                .map(|x| N::from_real(*x)),
+        );
+        info.error_coefficients = VectorN::<N, U4>::from_iterator(
+            Self::error_coefficients()
+                .as_slice()
+                .iter()
+                .map(|x| N::from_real(*x)),
+        );
+        RK23 { info }
+    }
+}
+
+impl<N: ComplexField, S: DimName> RungeKuttaSolver<N, S, U4> for RK23<N, S>
+where
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U4>,
+    DefaultAllocator: Allocator<N, U4, U4>,
+    DefaultAllocator: Allocator<N::RealField, U4>,
+    DefaultAllocator: Allocator<N::RealField, U4, U4>,
+    DefaultAllocator: Allocator<N, S, U4>,
+{
+    fn t_coefficients() -> VectorN<N::RealField, U4> {
+        VectorN::<N::RealField, U4>::from_column_slice(&[
+            N::RealField::from_f64(0.0).unwrap(),
+            N::RealField::from_f64(0.5).unwrap(),
+            N::RealField::from_f64(0.75).unwrap(),
+            N::RealField::from_f64(1.0).unwrap(),
+        ])
+    }
+
+    fn k_coefficients() -> MatrixMN<N::RealField, U4, U4> {
+        MatrixMN::<N::RealField, U4, U4>::new(
+            // Row 0
+            N::RealField::zero(),
+            N::RealField::zero(),
+            N::RealField::zero(),
+            N::RealField::zero(),
+            // Row 1
+            N::RealField::from_f64(0.5).unwrap(),
+            N::RealField::zero(),
+            N::RealField::zero(),
+            N::RealField::zero(),
+            // Row 2
+            N::RealField::zero(),
+            N::RealField::from_f64(0.75).unwrap(),
+            N::RealField::zero(),
+            N::RealField::zero(),
+            // Row 3
+            N::RealField::from_f64(2.0 / 9.0).unwrap(),
+            N::RealField::from_f64(1.0 / 3.0).unwrap(),
+            N::RealField::from_f64(4.0 / 9.0).unwrap(),
+            N::RealField::zero(),
+        )
+    }
+
+    fn avg_coefficients() -> VectorN<N::RealField, U4> {
+        VectorN::<N::RealField, U4>::from_column_slice(&[
+            N::RealField::from_f64(2.0 / 9.0).unwrap(),
+            N::RealField::from_f64(1.0 / 3.0).unwrap(),
+            N::RealField::from_f64(4.0 / 9.0).unwrap(),
+            N::RealField::zero(),
+        ])
+    }
+
+    fn error_coefficients() -> VectorN<N::RealField, U4> {
+        VectorN::<N::RealField, U4>::from_column_slice(&[
+            N::RealField::from_f64(2.0 / 9.0 - 7.0 / 24.0).unwrap(),
+            N::RealField::from_f64(1.0 / 3.0 - 0.25).unwrap(),
+            N::RealField::from_f64(4.0 / 9.0 - 1.0 / 3.0).unwrap(),
+            N::RealField::from_f64(-1.0 / 8.0).unwrap(),
+        ])
+    }
+
+    fn solve_ivp<
+        T: Clone,
+        F: FnMut(N::RealField, &[N], &mut T) -> Result<VectorN<N, S>, String>,
+    >(
+        self,
+        f: F,
+        params: &mut T,
+    ) -> super::Path<N, N::RealField, S> {
+        self.info.solve_ivp(f, params)
+    }
+
+    fn with_tolerance(mut self, tol: N::RealField) -> Result<Self, String> {
+        self.info = self.info.with_tolerance(tol)?;
+        Ok(self)
+    }
+
+    fn with_dt_max(mut self, max: N::RealField) -> Result<Self, String> {
+        self.info = self.info.with_dt_max(max)?;
+        Ok(self)
+    }
+
+    fn with_dt_min(mut self, min: N::RealField) -> Result<Self, String> {
+        self.info = self.info.with_dt_min(min)?;
+        Ok(self)
+    }
+
+    fn with_start(mut self, t_initial: N::RealField) -> Result<Self, String> {
+        self.info = self.info.with_start(t_initial)?;
+        Ok(self)
+    }
+
+    fn with_end(mut self, t_final: N::RealField) -> Result<Self, String> {
+        self.info = self.info.with_end(t_final)?;
+        Ok(self)
+    }
+
+    fn with_initial_conditions(mut self, start: &[N]) -> Result<Self, String> {
+        self.info = self.info.with_initial_conditions(start)?;
+        Ok(self)
+    }
+
+    fn build(self) -> Self {
+        self
+    }
+}
+
+impl<N: ComplexField, S: DimName> From<RK23<N, S>> for RKInfo<N, S, U4>
+where
+    DefaultAllocator: Allocator<N, S>,
+    DefaultAllocator: Allocator<N, U4>,
+    DefaultAllocator: Allocator<N, S, U4>,
+    DefaultAllocator: Allocator<N, U6, U4>,
+    DefaultAllocator: Allocator<N::RealField, U4>,
+    DefaultAllocator: Allocator<N::RealField, U4>,
+{
+    fn from(rk: RK23<N, S>) -> RKInfo<N, S, U4> {
         rk.info
     }
 }
