@@ -1,13 +1,12 @@
 use crate::polynomial::Polynomial;
-use nalgebra::{ComplexField, DMatrix, DVector, RealField, SVector};
+use nalgebra::{
+    allocator::Allocator, ComplexField, DMatrix, DVector, DefaultAllocator, DimName, RealField,
+    VectorN,
+};
 use num_traits::{FromPrimitive, One, Zero};
 
 /// Linear least-squares regression
-pub fn linear_fit<N>(xs: &[N], ys: &[N]) -> Result<Polynomial<N>, String>
-where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+pub fn linear_fit<N: ComplexField>(xs: &[N], ys: &[N]) -> Result<Polynomial<N>, String> {
     if xs.len() != ys.len() {
         return Err("linear_fit: xs length does not match ys length".to_owned());
     }
@@ -35,16 +34,14 @@ where
 }
 
 // Compute the J matrix for LM using finite differences, 3 point formula
-fn jac_finite_differences<N, F, const V: usize>(
+fn jac_finite_differences<N: ComplexField, V: DimName, F: FnMut(N, &VectorN<N, V>) -> N>(
     mut f: F,
     xs: &[N],
-    params: &mut SVector<N, V>,
+    params: &mut VectorN<N, V>,
     mat: &mut DMatrix<N>,
     h: N::RealField,
 ) where
-    N: ComplexField + FromPrimitive + Copy,
-    F: FnMut(N, &SVector<N, V>) -> N,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
+    DefaultAllocator: Allocator<N, V>,
 {
     let h = N::from_real(h);
     let denom = N::one() / (N::from_i32(2).unwrap() * h);
@@ -62,14 +59,13 @@ fn jac_finite_differences<N, F, const V: usize>(
 }
 
 // Compute the J matrix for LM using analytic formula
-fn jac_analytic<N, F, const V: usize>(
+fn jac_analytic<N: ComplexField, V: DimName, F: FnMut(N, &VectorN<N, V>) -> VectorN<N, V>>(
     mut jac: F,
     xs: &[N],
-    params: &mut SVector<N, V>,
+    params: &mut VectorN<N, V>,
     mat: &mut DMatrix<N>,
 ) where
-    N: ComplexField + Copy,
-    F: FnMut(N, &SVector<N, V>) -> SVector<N, V>,
+    DefaultAllocator: Allocator<N, V>,
 {
     for row in 0..mat.column(0).len() {
         let deriv = jac(xs[row], &params);
@@ -80,6 +76,7 @@ fn jac_analytic<N, F, const V: usize>(
 }
 
 #[derive(Debug, Clone)]
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
 pub struct CurveFitParams<N: ComplexField> {
     pub damping: N::RealField,
     pub tolerance: N::RealField,
@@ -87,13 +84,13 @@ pub struct CurveFitParams<N: ComplexField> {
     pub damping_mult: N::RealField,
 }
 
-impl<N: ComplexField + FromPrimitive> Default for CurveFitParams<N> {
+impl<N: ComplexField> Default for CurveFitParams<N> {
     fn default() -> Self {
         CurveFitParams {
-            damping: N::from_f64(2.0).unwrap().real(),
-            tolerance: N::from_f64(1e-5).unwrap().real(),
-            h: N::from_f64(0.1).unwrap().real(),
-            damping_mult: N::from_f64(1.5).unwrap().real(),
+            damping: N::RealField::from_f64(2.0).unwrap(),
+            tolerance: N::RealField::from_f64(1e-5).unwrap(),
+            h: N::RealField::from_f64(0.1).unwrap(),
+            damping_mult: N::RealField::from_f64(1.5).unwrap(),
         }
     }
 }
@@ -104,17 +101,15 @@ impl<N: ComplexField + FromPrimitive> Default for CurveFitParams<N> {
 /// can be found analytically, then use curve_fit_jac. Keeps iterating until
 /// the differences between the sum of the square residuals of two iterations
 /// is under tol.
-pub fn curve_fit<N, F, const V: usize>(
+pub fn curve_fit<N: ComplexField, V: DimName, F: FnMut(N, &VectorN<N, V>) -> N>(
     mut f: F,
     xs: &[N],
     ys: &[N],
     initial: &[N],
     params: &CurveFitParams<N>,
-) -> Result<SVector<N, V>, String>
+) -> Result<VectorN<N, V>, String>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-    F: FnMut(N, &SVector<N, V>) -> N,
+    DefaultAllocator: Allocator<N, V>,
 {
     let tol = params.tolerance;
     let mut damping = params.damping;
@@ -137,7 +132,7 @@ where
         return Err("curve_fit: xs length must match ys length".to_owned());
     }
 
-    let mut params = SVector::<N, V>::from_column_slice(initial);
+    let mut params = VectorN::<N, V>::from_column_slice(initial);
     let ys = DVector::<N>::from_column_slice(ys);
     let mut jac: DMatrix<N> = DMatrix::identity(xs.len(), params.len());
     jac_finite_differences(&mut f, xs, &mut params, &mut jac, h);
@@ -189,7 +184,7 @@ where
     }
 
     let mut last_sum_sq = sum_sq;
-    sum_sq += N::from_i32(2).unwrap().real() * tol;
+    sum_sq += N::RealField::from_i32(2).unwrap() * tol;
     while (last_sum_sq - sum_sq).abs() > tol {
         last_sum_sq = sum_sq;
         // Get right side of iteration equation
@@ -268,19 +263,21 @@ where
 /// the differences between the sum of the square residuals of two iterations
 /// is under tol. Jacobian should be a function that returns a column vector
 /// where jacobian[i] is the partial derivative of f with respect to param[i].
-pub fn curve_fit_jac<N, F, G, const V: usize>(
+pub fn curve_fit_jac<
+    N: ComplexField,
+    V: DimName,
+    F: FnMut(N, &VectorN<N, V>) -> N,
+    G: FnMut(N, &VectorN<N, V>) -> VectorN<N, V>,
+>(
     mut f: F,
     xs: &[N],
     ys: &[N],
     initial: &[N],
     mut jacobian: G,
     params: &CurveFitParams<N>,
-) -> Result<SVector<N, V>, String>
+) -> Result<VectorN<N, V>, String>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-    F: FnMut(N, &SVector<N, V>) -> N,
-    G: FnMut(N, &SVector<N, V>) -> SVector<N, V>,
+    DefaultAllocator: Allocator<N, V>,
 {
     let tol = params.tolerance;
     let mut damping = params.damping;
@@ -298,7 +295,7 @@ where
         return Err("curve_fit_jac: xs length must match ys length".to_owned());
     }
 
-    let mut params = SVector::<N, V>::from_column_slice(initial);
+    let mut params = VectorN::<N, V>::from_column_slice(initial);
     let ys = DVector::<N>::from_column_slice(ys);
     let mut jac: DMatrix<N> = DMatrix::identity(xs.len(), params.len());
     jac_analytic(&mut jacobian, xs, &mut params, &mut jac);
@@ -358,7 +355,7 @@ where
     }
 
     let mut last_sum_sq = sum_sq;
-    sum_sq += N::from_i32(2).unwrap().real() * tol;
+    sum_sq += N::RealField::from_i32(2).unwrap() * tol;
     while (last_sum_sq - sum_sq).abs() > tol {
         last_sum_sq = sum_sq;
         // Get right side of iteration equation
