@@ -7,10 +7,8 @@ use std::{any::TypeId, f64, iter::FromIterator, ops};
 
 /// Polynomial on a ComplexField.
 #[derive(Debug, Clone)]
-pub struct Polynomial<N: ComplexField + FromPrimitive + Copy>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+pub struct Polynomial<N: ComplexField> {
     // Index 0 is constant, 1 is linear, etc.
     coefficients: Vec<N>,
     tolerance: <N as ComplexField>::RealField,
@@ -23,10 +21,7 @@ macro_rules! polynomial {
   }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> Polynomial<N> {
     /// Returns the zero polynomial on a given field
     pub fn new() -> Self {
         Polynomial {
@@ -481,7 +476,7 @@ fn bit_reverse(mut k: usize, num_bits: usize) -> usize {
 }
 
 // Assumes vec is a power of 2 length
-fn bit_reverse_copy<N: RealField + Copy>(vec: &[Complex<N>]) -> Vec<Complex<N>> {
+fn bit_reverse_copy<N: RealField>(vec: &[Complex<N>]) -> Vec<Complex<N>> {
     let len = vec.len();
     let mut result = vec![Complex::new(N::zero(), N::zero()); len];
     let num_bits = (len as f64).log2() as usize;
@@ -491,10 +486,7 @@ fn bit_reverse_copy<N: RealField + Copy>(vec: &[Complex<N>]) -> Vec<Complex<N>>
     result
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> FromIterator<N> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> FromIterator<N> for Polynomial<N> {
     fn from_iter<I: IntoIterator<Item = N>>(iter: I) -> Polynomial<N> {
         Polynomial {
             coefficients: Vec::from_iter(iter),
@@ -503,19 +495,13 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> Default for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> Default for Polynomial<N> {
     fn default() -> Self {
         Self::new()
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> Zero for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> Zero for Polynomial<N> {
     fn zero() -> Polynomial<N> {
         Polynomial::new()
     }
@@ -532,10 +518,7 @@ where
 
 // Operator overloading
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Add<N> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Add<N> for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn add(mut self, rhs: N) -> Polynomial<N> {
@@ -544,10 +527,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Add<N> for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Add<N> for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn add(self, rhs: N) -> Polynomial<N> {
@@ -560,10 +540,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Add<Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Add<Polynomial<N>> for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn add(mut self, rhs: Polynomial<N>) -> Polynomial<N> {
@@ -580,10 +557,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Add<&Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Add<&Polynomial<N>> for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn add(mut self, rhs: &Polynomial<N>) -> Polynomial<N> {
@@ -601,10 +575,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Add<Polynomial<N>> for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Add<Polynomial<N>> for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn add(self, rhs: Polynomial<N>) -> Polynomial<N> {
@@ -631,10 +602,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Add<&Polynomial<N>> for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Add<&Polynomial<N>> for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn add(self, rhs: &Polynomial<N>) -> Polynomial<N> {
@@ -661,19 +629,13 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::AddAssign<N> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::AddAssign<N> for Polynomial<N> {
     fn add_assign(&mut self, rhs: N) {
         self.coefficients[0] += rhs;
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::AddAssign<Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::AddAssign<Polynomial<N>> for Polynomial<N> {
     fn add_assign(&mut self, rhs: Polynomial<N>) {
         let min_order = self.coefficients.len().min(rhs.coefficients.len());
         for (ind, val) in self.coefficients.iter_mut().take(min_order).enumerate() {
@@ -686,10 +648,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::AddAssign<&Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::AddAssign<&Polynomial<N>> for Polynomial<N> {
     fn add_assign(&mut self, rhs: &Polynomial<N>) {
         let min_order = self.coefficients.len().min(rhs.coefficients.len());
         for (ind, val) in self.coefficients.iter_mut().take(min_order).enumerate() {
@@ -702,10 +661,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Sub<N> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Sub<N> for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn sub(mut self, rhs: N) -> Polynomial<N> {
@@ -714,10 +670,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Sub<N> for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Sub<N> for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn sub(self, rhs: N) -> Polynomial<N> {
@@ -730,10 +683,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Sub<Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Sub<Polynomial<N>> for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn sub(mut self, rhs: Polynomial<N>) -> Polynomial<N> {
@@ -750,10 +700,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Sub<Polynomial<N>> for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Sub<Polynomial<N>> for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn sub(self, rhs: Polynomial<N>) -> Polynomial<N> {
@@ -780,10 +727,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Sub<&Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Sub<&Polynomial<N>> for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn sub(mut self, rhs: &Polynomial<N>) -> Polynomial<N> {
@@ -800,10 +744,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Sub<&Polynomial<N>> for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Sub<&Polynomial<N>> for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn sub(self, rhs: &Polynomial<N>) -> Polynomial<N> {
@@ -830,19 +771,13 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::SubAssign<N> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::SubAssign<N> for Polynomial<N> {
     fn sub_assign(&mut self, rhs: N) {
         self.coefficients[0] -= rhs;
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::SubAssign<Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::SubAssign<Polynomial<N>> for Polynomial<N> {
     fn sub_assign(&mut self, rhs: Polynomial<N>) {
         let min_order = self.coefficients.len().min(rhs.coefficients.len());
         for (ind, val) in self.coefficients.iter_mut().take(min_order).enumerate() {
@@ -855,10 +790,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::SubAssign<&Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::SubAssign<&Polynomial<N>> for Polynomial<N> {
     fn sub_assign(&mut self, rhs: &Polynomial<N>) {
         let min_order = self.coefficients.len().min(rhs.coefficients.len());
         for (ind, val) in self.coefficients.iter_mut().take(min_order).enumerate() {
@@ -871,10 +803,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Mul<N> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Mul<N> for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn mul(mut self, rhs: N) -> Polynomial<N> {
@@ -885,10 +814,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Mul<N> for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Mul<N> for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn mul(self, rhs: N) -> Polynomial<N> {
@@ -903,13 +829,7 @@ where
     }
 }
 
-fn multiply<N: ComplexField + FromPrimitive + Copy>(
-    lhs: &Polynomial<N>,
-    rhs: &Polynomial<N>,
-) -> Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+fn multiply<N: ComplexField>(lhs: &Polynomial<N>, rhs: &Polynomial<N>) -> Polynomial<N> {
     // Do scalar multiplication if one side has no powers
     if rhs.coefficients.len() == 1 {
         return lhs * rhs.coefficients[0];
@@ -941,10 +861,7 @@ where
     Polynomial::<N>::idft(&product_points, lhs.tolerance)
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Mul<Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Mul<Polynomial<N>> for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn mul(self, rhs: Polynomial<N>) -> Polynomial<N> {
@@ -952,10 +869,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Mul<&Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Mul<&Polynomial<N>> for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn mul(self, rhs: &Polynomial<N>) -> Polynomial<N> {
@@ -963,10 +877,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Mul<Polynomial<N>> for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Mul<Polynomial<N>> for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn mul(self, rhs: Polynomial<N>) -> Polynomial<N> {
@@ -974,10 +885,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Mul<&Polynomial<N>> for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Mul<&Polynomial<N>> for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn mul(self, rhs: &Polynomial<N>) -> Polynomial<N> {
@@ -985,10 +893,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::MulAssign<N> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::MulAssign<N> for Polynomial<N> {
     fn mul_assign(&mut self, rhs: N) {
         for val in self.coefficients.iter_mut() {
             *val *= rhs;
@@ -996,28 +901,19 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::MulAssign<Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::MulAssign<Polynomial<N>> for Polynomial<N> {
     fn mul_assign(&mut self, rhs: Polynomial<N>) {
         self.coefficients = multiply(&self, &rhs).coefficients;
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::MulAssign<&Polynomial<N>> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::MulAssign<&Polynomial<N>> for Polynomial<N> {
     fn mul_assign(&mut self, rhs: &Polynomial<N>) {
         self.coefficients = multiply(&self, &rhs).coefficients;
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Div<N> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Div<N> for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn div(mut self, rhs: N) -> Polynomial<N> {
@@ -1028,10 +924,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Div<N> for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Div<N> for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn div(self, rhs: N) -> Polynomial<N> {
@@ -1046,10 +939,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::DivAssign<N> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::DivAssign<N> for Polynomial<N> {
     fn div_assign(&mut self, rhs: N) {
         for val in &mut self.coefficients {
             *val /= rhs;
@@ -1057,10 +947,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Neg for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Neg for Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn neg(mut self) -> Polynomial<N> {
@@ -1071,10 +958,7 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> ops::Neg for &Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> ops::Neg for &Polynomial<N> {
     type Output = Polynomial<N>;
 
     fn neg(self) -> Polynomial<N> {
@@ -1085,16 +969,13 @@ where
     }
 }
 
-impl<N: ComplexField + FromPrimitive + Copy> From<N> for Polynomial<N>
-where
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-{
+impl<N: ComplexField> From<N> for Polynomial<N> {
     fn from(n: N) -> Polynomial<N> {
         polynomial![n]
     }
 }
 
-impl<N: RealField + FromPrimitive + Copy> From<Polynomial<N>> for Polynomial<Complex<N>> {
+impl<N: RealField> From<Polynomial<N>> for Polynomial<Complex<N>> {
     fn from(poly: Polynomial<N>) -> Polynomial<Complex<N>> {
         poly.make_complex()
     }
