@@ -4,8 +4,12 @@
  * See repository LICENSE for information.
  */
 
-use nalgebra::{ComplexField, Const, DimMin, RealField, SMatrix, SVector};
-use num_traits::{FromPrimitive, Zero};
+use nalgebra::{
+    allocator::Allocator,
+    dimension::{DimMin, DimMinimum},
+    ComplexField, DefaultAllocator, DimName, MatrixN, RealField, VectorN, U1,
+};
+use num_traits::Zero;
 
 mod polynomial;
 pub use polynomial::*;
@@ -40,16 +44,12 @@ pub use polynomial::*;
 ///   let solution = bisection((-1.0, 1.0), cubic, 0.001, 1000).unwrap();
 /// }
 /// ```
-pub fn bisection<N, F>(
+pub fn bisection<N: RealField, F: FnMut(N) -> N>(
     (mut left, mut right): (N, N),
     mut f: F,
     tol: N,
     n_max: usize,
-) -> Result<N, String>
-where
-    N: RealField + FromPrimitive + Copy,
-    F: FnMut(N) -> N,
-{
+) -> Result<N, String> {
     if left >= right {
         return Err("Bisection: requirement: right > left".to_owned());
     }
@@ -122,17 +122,19 @@ where
 ///   let solution = steffensen(0.5f64, cosine, 0.0001, 1000)?;
 ///   Ok(())
 /// }
-pub fn steffensen<N>(mut initial: N, f: fn(N) -> N, tol: N, n_max: usize) -> Result<N, String>
-where
-    N: RealField + FromPrimitive + Copy,
-{
+pub fn steffensen<N: RealField + From<f64> + Copy>(
+    mut initial: N,
+    f: fn(N) -> N,
+    tol: N,
+    n_max: usize,
+) -> Result<N, String> {
     let mut n = 0;
 
     while n < n_max {
         let guess = f(initial);
         let new_guess = f(guess);
-        let diff = initial
-            - (guess - initial).powi(2) / (new_guess - N::from_f64(2.0).unwrap() * guess + initial);
+        let diff =
+            initial - (guess - initial).powi(2) / (new_guess - N::from(2.0) * guess + initial);
         if (diff - initial).abs() <= tol {
             return Ok(diff);
         }
@@ -166,35 +168,38 @@ where
 ///
 /// # Examples
 /// ```
-/// use nalgebra::{SVector, SMatrix};
+/// use nalgebra::{VectorN, U1, MatrixN};
 /// use bacon_sci::roots::newton;
-/// fn cubic(x: &[f64]) -> SVector<f64, 1> {
-///   SVector::<f64, 1>::from_iterator(x.iter().map(|x| x.powi(3)))
+/// fn cubic(x: &[f64]) -> VectorN<f64, U1> {
+///   VectorN::<f64, U1>::from_iterator(x.iter().map(|x| x.powi(3)))
 /// }
 ///
-/// fn cubic_deriv(x: &[f64]) -> SMatrix<f64, 1, 1> {
-///  SMatrix::<f64, 1, 1>::from_iterator(x.iter().map(|x| 3.0*x.powi(2)))
+/// fn cubic_deriv(x: &[f64]) -> MatrixN<f64, U1> {
+///  MatrixN::<f64, U1>::from_iterator(x.iter().map(|x| 3.0*x.powi(2)))
 /// }
 /// //...
 /// fn example() {
 ///   let solution = newton(&[0.1], cubic, cubic_deriv, 0.001, 1000).unwrap();
 /// }
 /// ```
-pub fn newton<N, F, G, const S: usize>(
+pub fn newton<N, S, F, G>(
     initial: &[N],
     mut f: F,
     mut jac: G,
     tol: <N as ComplexField>::RealField,
     n_max: usize,
-) -> Result<SVector<N, S>, String>
+) -> Result<VectorN<N, S>, String>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-    F: FnMut(&[N]) -> SVector<N, S>,
-    G: FnMut(&[N]) -> SMatrix<N, S, S>,
-    Const<S>: DimMin<Const<S>, Output = Const<S>>,
+    N: ComplexField,
+    S: DimName + DimMin<S, Output = S>,
+    F: FnMut(&[N]) -> VectorN<N, S>,
+    G: FnMut(&[N]) -> MatrixN<N, S>,
+    DefaultAllocator: Allocator<N, S>
+        + Allocator<N, S, S>
+        + Allocator<(usize, usize), DimMinimum<S, S>>
+        + Allocator<(usize, usize), S>,
 {
-    let mut guess = SVector::<N, S>::from_column_slice(initial);
+    let mut guess = VectorN::<N, S>::from_column_slice(initial);
     let mut norm = guess.dot(&guess).sqrt().abs();
     let mut n = 0;
 
@@ -225,17 +230,18 @@ where
     Err("Newton: Maximum iterations exceeded".to_owned())
 }
 
-fn jac_finite_diff<N, F, const S: usize>(
+fn jac_finite_diff<N, S, F>(
     mut f: F,
-    x: &mut SVector<N, S>,
+    x: &mut VectorN<N, S>,
     h: <N as ComplexField>::RealField,
-) -> SMatrix<N, S, S>
+) -> MatrixN<N, S>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-    F: FnMut(&[N]) -> SVector<N, S>,
+    N: ComplexField,
+    S: DimName,
+    F: FnMut(&[N]) -> VectorN<N, S>,
+    DefaultAllocator: Allocator<N, S> + Allocator<N, S, S>,
 {
-    let mut mat = SMatrix::<N, S, S>::zero();
+    let mut mat = MatrixN::<N, S>::zero();
     let h = N::from_real(h);
     let denom = N::one() / (N::from_i32(2).unwrap() * h);
 
@@ -276,32 +282,33 @@ where
 ///
 /// # Examples
 /// ```
-/// use nalgebra::SVector;
+/// use nalgebra::{VectorN, U1};
 /// use bacon_sci::roots::secant;
-/// fn cubic(x: &[f64]) -> SVector<f64, 1> {
-///   SVector::<f64, 1>::from_iterator(x.iter().map(|x| x.powi(3)))
+/// fn cubic(x: &[f64]) -> VectorN<f64, U1> {
+///   VectorN::<f64, U1>::from_iterator(x.iter().map(|x| x.powi(3)))
 /// }
 /// //...
 /// fn example() {
 ///   let solution = secant(&[0.1], cubic, 0.1, 0.001, 1000).unwrap();
 /// }
 /// ```
-pub fn secant<N, F, const S: usize>(
+pub fn secant<N, S, F>(
     initial: &[N],
     mut func: F,
     h: <N as ComplexField>::RealField,
     tol: <N as ComplexField>::RealField,
     n_max: usize,
-) -> Result<SVector<N, S>, String>
+) -> Result<VectorN<N, S>, String>
 where
-    N: ComplexField + FromPrimitive + Copy,
-    <N as ComplexField>::RealField: FromPrimitive + Copy,
-    F: FnMut(&[N]) -> SVector<N, S>,
-    Const<S>: DimMin<Const<S>, Output = Const<S>>,
+    N: ComplexField,
+    S: DimName + DimMin<S, Output = S>,
+    F: FnMut(&[N]) -> VectorN<N, S>,
+    DefaultAllocator:
+        Allocator<N, S> + Allocator<N, S, S> + Allocator<(usize, usize), S> + Allocator<N, U1, S>,
 {
     let mut n = 2;
 
-    let mut guess = SVector::<N, S>::from_column_slice(initial);
+    let mut guess = VectorN::<N, S>::from_column_slice(initial);
     let mut func_eval = func(guess.as_slice());
 
     let jac = jac_finite_diff(&mut func, &mut guess, h);
@@ -352,11 +359,11 @@ where
 ///   let solution = brent((0.1, -0.1), cubic, 1e-5).unwrap();
 /// }
 /// ```
-pub fn brent<N, F>(initial: (N, N), mut f: F, tol: N) -> Result<N, String>
-where
-    N: RealField + FromPrimitive + Copy,
-    F: FnMut(N) -> N,
-{
+pub fn brent<N: RealField, F: FnMut(N) -> N>(
+    initial: (N, N),
+    mut f: F,
+    tol: N,
+) -> Result<N, String> {
     if !tol.is_sign_positive() {
         return Err("brent: tolerance must be positive".to_owned());
     }
@@ -454,11 +461,14 @@ where
 ///   let solution = itp((0.1, -0.1), cubic, 0.1, 2.0, 0.99, 1e-5).unwrap();
 /// }
 /// ```
-pub fn itp<N, F>(initial: (N, N), mut f: F, k_1: N, k_2: N, n_0: N, tol: N) -> Result<N, String>
-where
-    N: RealField + FromPrimitive + Copy,
-    F: FnMut(N) -> N,
-{
+pub fn itp<N: RealField, F: FnMut(N) -> N>(
+    initial: (N, N),
+    mut f: F,
+    k_1: N,
+    k_2: N,
+    n_0: N,
+    tol: N,
+) -> Result<N, String> {
     if !tol.is_sign_positive() {
         return Err("itp: tolerance must be positive".to_owned());
     }
